@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2023  Błażej Szczygieł
+    Copyright (C) 2010-2025  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -23,6 +23,7 @@
 #include <Frame.hpp>
 #include <Version.hpp>
 #include <Reader.hpp>
+#include <StreamInfo.hpp>
 
 #include <QGraphicsDropShadowEffect>
 #include <QGraphicsBlurEffect>
@@ -38,6 +39,7 @@
 #include <QTextCodec>
 #include <QMessageBox>
 #include <QStyleOption>
+#include <QInputDialog>
 #include <QGuiApplication>
 #include <QRegularExpression>
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
@@ -151,7 +153,7 @@ QString Functions::Url(QString url, const QString &pth)
             url.prepend(addPth);
         }
 #ifndef Q_OS_WIN
-        if (hasBackslash && !QFileInfo(url).exists())
+        if (hasBackslash && !QFileInfo::exists(url))
             url.replace("\\", "/");
 #endif
         url.prepend("file://");
@@ -195,17 +197,23 @@ QString Functions::filePath(const QString &f)
 QString Functions::fileName(QString f, bool extension)
 {
     /* Tylko dla adresów do przekonwertowania */
-    QString real_url;
-    if (splitPrefixAndUrlIfHasPluginPrefix(f, nullptr, &real_url))
+    QString real_url, addressPrefixName;
+    if (splitPrefixAndUrlIfHasPluginPrefix(f, &addressPrefixName, &real_url))
     {
         if (real_url.startsWith("file://"))
             return fileName(real_url, extension);
+        if (!extension && isResourcePlaylist(f))
+            return fileName(QStringLiteral("QMPlay2://") + real_url, false);
         return real_url;
     }
     /**/
 #ifndef Q_OS_WIN
     if (f == "file:///")
         return "/";
+#endif
+#ifdef Q_OS_ANDROID
+    if (!extension && f.startsWith("content://"))
+        return QFileInfo(f).completeBaseName();
 #endif
     while (f.endsWith("/"))
         f.chop(1);
@@ -388,11 +396,12 @@ bool Functions::mustRepaintOSD(const QMPlay2OSDList &osd_list, const OsdIdList &
             osd->iterate([&](const QMPlay2OSD::Image &img) {
                 if (osd->needsRescale())
                 {
+                    const auto imgRect = osd->getRect(img);
                     *bounds |= QRectF(
-                        img.rect.x() * *scaleW,
-                        img.rect.y() * *scaleH,
-                        img.rect.width() * *scaleW,
-                        img.rect.height() * *scaleH
+                        imgRect.x() * *scaleW,
+                        imgRect.y() * *scaleH,
+                        imgRect.width() * *scaleW,
+                        imgRect.height() * *scaleH
                     ).toAlignedRect();
                 }
                 else
@@ -428,7 +437,7 @@ void Functions::paintOSD(bool rgbSwapped, const QMPlay2OSDList &osd_list, const
             );
             if (osd->needsRescale())
             {
-                painter.drawImage(img.rect, qImg);
+                painter.drawImage(osd->getRect(img), qImg);
             }
             else
             {
@@ -571,6 +580,11 @@ int Functions::scaleEQValue(int val, int min, int max)
 
 QByteArray Functions::convertToASS(QString txt)
 {
+    txt.replace("&nbsp;", " ", Qt::CaseInsensitive);
+    txt.replace("&lt;", "<", Qt::CaseInsensitive);
+    txt.replace("&gt;", ">", Qt::CaseInsensitive);
+    txt.replace("&amp;", "&", Qt::CaseInsensitive);
+    txt.replace("&quot;", "\"", Qt::CaseInsensitive);
     txt.replace("<i>", "{\\i1}", Qt::CaseInsensitive);
     txt.replace("</i>", "{\\i0}", Qt::CaseInsensitive);
     txt.replace("<b>", "{\\b1}", Qt::CaseInsensitive);
@@ -629,7 +643,7 @@ QStringList Functions::getUrlsFromMimeData(const QMimeData *mimeData, const bool
     }
     else if (mimeData->hasText())
     {
-        urls = mimeData->text().remove('\r').split('\n', QT_SKIP_EMPTY_PARTS);
+        urls = mimeData->text().remove('\r').split('\n', Qt::SkipEmptyParts);
     }
     if (checkExtensionsForUrl)
     {
@@ -710,6 +724,13 @@ void Functions::getDataIfHasPluginPrefix(const QString &entireUrl, QString *url,
         }
     }
 }
+bool Functions::isResourcePlaylist(const QString &url)
+{
+    QString realUrl, addressPrefixName;
+    if (!Functions::splitPrefixAndUrlIfHasPluginPrefix(url, &addressPrefixName, &realUrl))
+        return false;
+    return (addressPrefixName == QStringLiteral("QMPlay2")) && realUrl.endsWith(".pls");
+}
 
 void Functions::hFlip(quint8 *data, int linesize, int height, int width)
 {
@@ -819,13 +840,25 @@ QString Functions::prepareFFmpegUrl(QString url, AVDictionary *&options, bool de
     else
     {
         const QByteArray cookies = setCookies ? QMPlay2Core.getCookies(url) : QByteArray();
-        const QByteArray rawHeaders = setRawHeaders ? QMPlay2Core.getRawHeaders(url) : QByteArray();
+        QByteArray rawHeaders = setRawHeaders ? QMPlay2Core.getRawHeaders(url) : QByteArray();
         const QByteArray userAgent = [&] {
             if (!userAgentArg.isNull())
                 return userAgentArg;
             return getUserAgent(defUserAgentWithMozilla);
         }();
 
+        if (!cookies.isEmpty())
+        {
+            if (const auto idx1 = rawHeaders.indexOf("Cookie: "); idx1 > -1)
+            {
+                if (const auto idx2 = rawHeaders.indexOf("\r\n", idx1 + 8); idx2 > -1)
+                {
+                    rawHeaders.remove(idx1, idx2 - idx1 + 2);
+                }
+            }
+            rawHeaders += QByteArray("Cookie: " + cookies + "\r\n");
+        }
+
         if (url.startsWith("mms:"))
             url.insert(3, 'h');
 
@@ -833,119 +866,14 @@ QString Functions::prepareFFmpegUrl(QString url, AVDictionary *&options, bool de
             av_dict_set(&options, "icy", icy ? "1" : "0", 0);
         av_dict_set(&options, "user_agent", userAgent, 0);
 
-        if (!cookies.isEmpty())
-            av_dict_set(&options, "headers", "Cookie: " + cookies + "\r\n", 0);
         if (!rawHeaders.isEmpty())
-            av_dict_set(&options, "headers", rawHeaders, 0);
+            av_dict_set(&options, "headers", rawHeaders.constData(), 0);
 
         av_dict_set(&options, "reconnect", "1", 0);
     }
     return url;
 }
 
-QByteArray Functions::decryptAes256Cbc(const QByteArray &password, const QByteArray &salt, const QByteArray &ciphered)
-{
-    using EVP_CIPHER_CTX = void;
-    using EVP_CIPHER = void;
-    using EVP_MD = void;
-    using ENGINE = void;
-
-    using EVP_md5_Type = const EVP_MD *(*)(void);
-    using EVP_aes_256_cbc_Type = const EVP_CIPHER *(*)(void);
-    using EVP_BytesToKey_Type = int(*)(const EVP_CIPHER *type, const EVP_MD *md, const unsigned char *salt, const unsigned char *data, int datal, int count, unsigned char *key, unsigned char *iv);
-    using EVP_CIPHER_CTX_init_Type = void(*)(EVP_CIPHER_CTX *a);
-    using EVP_CIPHER_CTX_new_Type = EVP_CIPHER_CTX *(*)();
-    using EVP_DecryptUpdate_Type = int(*)(EVP_CIPHER_CTX *ctx, unsigned char *out, int *outl, const unsigned char *in, int inl);
-    using EVP_DecryptInit_ex_Type = int(*)(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *cipher, ENGINE *impl, const unsigned char *key, const unsigned char *iv);
-    using EVP_DecryptFinal_ex_Type = int(*)(EVP_CIPHER_CTX *ctx, unsigned char *outm, int *outl);
-    using EVP_CIPHER_CTX_cleanup_Type = int(*)(EVP_CIPHER_CTX *a);
-    using EVP_CIPHER_CTX_reset_Type = int(*)(EVP_CIPHER_CTX *a);
-
-    QLibrary libssl;
-#if defined(Q_OS_WIN64)
-    libssl.setFileName("libcrypto-1_1-x64");
-    libssl.load();
-#elif defined(Q_OS_WIN32)
-    libssl.setFileName("libcrypto-1_1");
-    libssl.load();
-#else
-    libssl.setFileName("ssl");
-    if (!libssl.load())
-    {
-        libssl.setFileNameAndVersion("ssl", "1.1");
-        if (!libssl.load())
-        {
-            libssl.setFileNameAndVersion("ssl", "1.0.0");
-            libssl.load();
-        }
-    }
-#endif
-    if (!libssl.isLoaded())
-    {
-        QMPlay2Core.logError("Cannot load OpenSSL library", true, true);
-        return QByteArray();
-    }
-
-    EVP_md5_Type EVP_md5 = (EVP_md5_Type)libssl.resolve("EVP_md5");
-    EVP_aes_256_cbc_Type EVP_aes_256_cbc = (EVP_aes_256_cbc_Type)libssl.resolve("EVP_aes_256_cbc");
-    EVP_BytesToKey_Type EVP_BytesToKey = (EVP_BytesToKey_Type)libssl.resolve("EVP_BytesToKey");
-    EVP_CIPHER_CTX_init_Type EVP_CIPHER_CTX_init = (EVP_CIPHER_CTX_init_Type)libssl.resolve("EVP_CIPHER_CTX_init");
-    EVP_CIPHER_CTX_new_Type EVP_CIPHER_CTX_new = (EVP_CIPHER_CTX_new_Type)libssl.resolve("EVP_CIPHER_CTX_new");
-    EVP_DecryptUpdate_Type EVP_DecryptUpdate = (EVP_DecryptUpdate_Type)libssl.resolve("EVP_DecryptUpdate");
-    EVP_DecryptInit_ex_Type EVP_DecryptInit_ex = (EVP_DecryptInit_ex_Type)libssl.resolve("EVP_DecryptInit_ex");
-    EVP_DecryptFinal_ex_Type EVP_DecryptFinal_ex = (EVP_DecryptFinal_ex_Type)libssl.resolve("EVP_DecryptFinal_ex");
-    EVP_CIPHER_CTX_cleanup_Type EVP_CIPHER_CTX_cleanup = (EVP_CIPHER_CTX_cleanup_Type)libssl.resolve("EVP_CIPHER_CTX_cleanup");
-    EVP_CIPHER_CTX_reset_Type EVP_CIPHER_CTX_reset = (EVP_CIPHER_CTX_reset_Type)libssl.resolve("EVP_CIPHER_CTX_reset");
-
-    if (!EVP_md5 || !EVP_aes_256_cbc || !EVP_BytesToKey || !EVP_DecryptUpdate || !EVP_DecryptInit_ex || !EVP_DecryptFinal_ex)
-    {
-        QMPlay2Core.logError("Cannot resolve OpenSSL methods", true, true);
-        return QByteArray();
-    }
-    if ((!EVP_CIPHER_CTX_init && !EVP_CIPHER_CTX_new) || (!EVP_CIPHER_CTX_cleanup && !EVP_CIPHER_CTX_reset))
-    {
-        QMPlay2Core.logError("Cannot resolve OpenSSL EVP new/reset methods", true, true);
-        return QByteArray();
-    }
-
-    quint8 key[32], iv[32];
-    const int keyLen = EVP_BytesToKey(EVP_aes_256_cbc(), EVP_md5(), (const quint8 *)salt.constData(), (const quint8 *)password.constData(), password.length(), 1, key, iv);
-    if (keyLen != 32)
-        return QByteArray();
-
-    const bool newLibrary = (EVP_CIPHER_CTX_new && EVP_CIPHER_CTX_reset);
-
-    int decryptedLen = ciphered.length(), finalizeLen = 0;
-    QByteArray deciphered(decryptedLen, Qt::Uninitialized);
-
-    EVP_CIPHER_CTX *ctx;
-    if (newLibrary)
-    {
-        ctx = EVP_CIPHER_CTX_new();
-    }
-    else
-    {
-        ctx = malloc(256);
-        EVP_CIPHER_CTX_init(ctx);
-    }
-    EVP_DecryptInit_ex(ctx, EVP_aes_256_cbc(), nullptr, key, iv);
-    EVP_DecryptInit_ex(ctx, nullptr, nullptr, nullptr, nullptr);
-    EVP_DecryptUpdate(ctx, (quint8 *)deciphered.data(), &decryptedLen, (const quint8 *)ciphered.constData(), decryptedLen);
-    EVP_DecryptFinal_ex(ctx, (quint8 *)deciphered.data() + decryptedLen, &finalizeLen);
-    if (newLibrary)
-    {
-        EVP_CIPHER_CTX_reset(ctx);
-    }
-    else
-    {
-        EVP_CIPHER_CTX_cleanup(ctx);
-        free(ctx);
-    }
-
-    deciphered.resize(decryptedLen + finalizeLen);
-    return deciphered;
-}
-
 QByteArray Functions::textWithFallbackEncoding(const QByteArray &data)
 {
     QTextCodec *codec = QTextCodec::codecForUtfText(data, QTextCodec::codecForName(QMPlay2Core.getSettings().getByteArray("FallbackSubtitlesEncoding")));
@@ -1073,7 +1001,7 @@ QMatrix4x4 Functions::getColorPrimariesTo709Matrix(const QVector2D &wp, array<QV
             Yr,   Yg,   Yb,   0.0f,
             Zr,   Zg,   Zb,   0.0f,
             0.0f, 0.0f, 0.0f, 1.0f
-        ).inverted() * wpXyz;
+        ).inverted().map(wpXyz);
 
         return QMatrix4x4(
             S[0] * Xr, S[1] * Xg, S[2] * Xb, 0.0f,
@@ -1109,12 +1037,6 @@ QMatrix4x4 Functions::getColorPrimariesTo709Matrix(AVColorPrimaries colorPrimari
     return getColorPrimariesTo709Matrix(wp, primaries);
 }
 
-bool Functions::isX11EGL()
-{
-    static bool isEGL = (QString(qgetenv("QT_XCB_GL_INTEGRATION")).compare("xcb_egl", Qt::CaseInsensitive) == 0);
-    return isEGL;
-}
-
 bool Functions::compareText(const QString &a, const QString &b)
 {
     QRegularExpression rx(R"(\d+)");
@@ -1159,10 +1081,10 @@ bool Functions::compareText(const QString &a, const QString &b)
                 }
             }
         }
-        return (newA < newB);
+        return (newA.toLower() < newB.toLower());
     }
 
-    return (a < b);
+    return (a.toLower() < b.toLower());
 }
 
 bool Functions::hasTouchScreen()
@@ -1199,3 +1121,100 @@ QString Functions::getBitrateStr(const int64_t bitRate)
         return QString("%1 kbps").arg(qRound64(bitRate / 1000.0));
     return QString("%1 Mbps").arg(bitRate / 1000000.0, 0, 'f', 3);
 }
+
+QString Functions::getSeqFile(const QString &dir, const QString &ext, const QString &frag)
+{
+    quint16 num = 0;
+    for (const QString &f : QDir(dir).entryList({QString("QMPlay2_%1_?????%2").arg(frag, ext)}, QDir::Files, QDir::Name))
+    {
+        const quint16 n = QStringView(f).mid(8 + frag.size() + 1, 5).toUShort();
+        if (n > num)
+            num = n;
+    }
+    return QString("QMPlay2_%1_%2%3").arg(frag).arg(++num, 5, 10, QChar('0')).arg(ext);
+}
+
+std::pair<QString, QString> Functions::determineExtFmt(const QList<StreamInfo *> &streamsInfo)
+{
+    QString ext("mkv");
+    QString fmt("matroska");
+
+    if (streamsInfo.size() == 1)
+    {
+        switch (streamsInfo.at(0)->params->codec_id)
+        {
+            case AV_CODEC_ID_NONE:
+                ext = fmt = "wav";
+                break;
+            case AV_CODEC_ID_MP3:
+                ext = fmt = "mp3";
+                break;
+            case AV_CODEC_ID_AAC:
+                ext = "aac";
+                fmt = "adts";
+                break;
+            case AV_CODEC_ID_AC3:
+                ext = fmt = "ac3";
+                break;
+            case AV_CODEC_ID_VORBIS:
+                ext = fmt = "ogg";
+                break;
+            case AV_CODEC_ID_OPUS:
+                ext = "opus";
+                fmt  = "ogg";
+                break;
+            case AV_CODEC_ID_H264:
+            case AV_CODEC_ID_HEVC:
+                ext = "ts";
+                fmt = "mpegts";
+                break;
+            default:
+                break;
+        }
+    }
+    else if (streamsInfo.size() == 2)
+    {
+        const QSet<AVCodecID> codecIDs {
+            streamsInfo.at(0)->params->codec_id,
+            streamsInfo.at(1)->params->codec_id,
+        };
+        if ((codecIDs.contains(AV_CODEC_ID_AAC) || codecIDs.contains(AV_CODEC_ID_MP3) || codecIDs.contains(AV_CODEC_ID_AC3))
+                && (codecIDs.contains(AV_CODEC_ID_H264) || codecIDs.contains(AV_CODEC_ID_HEVC)))
+        {
+            ext = "ts";
+            fmt = "mpegts";
+        }
+    }
+
+    return {ext, fmt};
+}
+
+void Functions::getUserDoubleValue(QWidget *parent, const QString &title, const QString &label, double value, double min, double max, int decimals, double step, const function<void(double)> &onChanged)
+{
+    Q_ASSERT(onChanged);
+
+    QInputDialog input(parent);
+    input.setInputMode(QInputDialog::DoubleInput);
+    input.setWindowTitle(title);
+    input.setTextValue(label);
+    input.setDoubleDecimals(decimals);
+    input.setDoubleStep(step);
+    input.setDoubleRange(min, max);
+    input.setDoubleValue(value);
+
+    QObject::connect(&input, &QInputDialog::doubleValueChanged, parent, [&onChanged](double newValue) {
+        if (qFuzzyIsNull(newValue))
+            onChanged(0.0);
+        else if (qFuzzyCompare(newValue, 1.0))
+            onChanged(1.0);
+        else
+            onChanged(newValue);
+    });
+
+    onChanged(value);
+
+    if (input.exec() != QDialog::Accepted)
+    {
+        onChanged(value);
+    }
+}
