@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2022  Błażej Szczygieł
+    Copyright (C) 2010-2025  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -19,6 +19,7 @@
 #pragma once
 
 #include <QMPlay2Lib.hpp>
+#include <PlaylistEntry.hpp>
 
 #include <QAtomicInt>
 #include <QObject>
@@ -59,8 +60,6 @@ public:
     };
 
 public:
-    using GroupEntries = QVector<QPair<QString, QString>>;
-
     static inline QMPlay2CoreClass &instance()
     {
         return *qmplay2Core;
@@ -79,7 +78,7 @@ public:
     static bool isGlOnWindowForced();
 #endif
 
-    void init(bool loadModules, bool modulesInSubdirs, const QString &libPath, const QString &sharePath, const QString &profileName);
+    void init(bool loadModulesAndGpu, bool modulesInSubdirs, const QString &libPath, const QString &sharePath, const QString &profileName);
     void quit();
 
     bool canSuspend();
@@ -113,12 +112,17 @@ public:
     {
         return *settings;
     }
+    inline Settings &getUrlPosSets() const
+    {
+        return *m_urlPosSets;
+    }
 
     qreal getVideoDevicePixelRatio() const;
 
     QIcon getIconFromTheme(const QString &iconName, const QIcon &fallback = QIcon());
 
     Q_SIGNAL void wallpaperChanged(bool hasWallpaper, double alpha);
+    Q_SIGNAL void wallpaperChanged(const QPixmap &wallpaper);
 
     Q_SIGNAL void dockVideo(QWidget *w);
 
@@ -207,7 +211,7 @@ public:
     void addDescriptionForUrl(const QString &url, const QString &description, const bool removeAfterUse = true);
     QString getDescriptionForUrl(const QString &url);
 
-    void loadPlaylistGroup(const QString &name, const GroupEntries &entries, bool enqueue = false);
+    QString writePlaylistResource(const QString &name, const QString &args, const PlaylistEntries &playlistEntries);
 
     QString rendererName() const;
     Renderer renderer() const;
@@ -242,6 +246,7 @@ protected:
 
     QIcon *qmplay2Icon;
     Settings *settings;
+    Settings *m_urlPosSets = nullptr;
 
 private:
     static QMPlay2CoreClass *qmplay2Core;
