@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2022  Błażej Szczygieł
+    Copyright (C) 2010-2025  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -103,15 +103,15 @@ void Slider::mousePressEvent(QMouseEvent *e)
     if (style->styleHint(QStyle::SH_Slider_PageSetButtons) & e->button())
     {
         const auto sliderAbsoluteSetButtons = static_cast<Qt::MouseButtons>(style->styleHint(QStyle::SH_Slider_AbsoluteSetButtons));
-        if (sliderAbsoluteSetButtons & (Qt::LeftButton | Qt::MidButton))
+        if (sliderAbsoluteSetButtons & (Qt::LeftButton | Qt::MiddleButton))
         {
             Qt::MouseButton sliderAbsoluteSetButton = Qt::NoButton;
             if (sliderAbsoluteSetButtons & Qt::LeftButton)
                 sliderAbsoluteSetButton = Qt::LeftButton;
-            else if (sliderAbsoluteSetButtons & Qt::MidButton)
-                sliderAbsoluteSetButton = Qt::MidButton;
+            else if (sliderAbsoluteSetButtons & Qt::MiddleButton)
+                sliderAbsoluteSetButton = Qt::MiddleButton;
 
-            QMouseEvent ev(e->type(), e->pos(), sliderAbsoluteSetButton, sliderAbsoluteSetButtons, e->modifiers());
+            QMouseEvent ev(e->type(), e->pos(), e->globalPos(), sliderAbsoluteSetButton, sliderAbsoluteSetButtons, e->modifiers());
             QSlider::mousePressEvent(&ev);
             e->setAccepted(ev.isAccepted());
 
@@ -151,15 +151,16 @@ void Slider::mouseMoveEvent(QMouseEvent *e)
 }
 void Slider::wheelEvent(QWheelEvent *e)
 {
+    const auto angleDelta  = e->angleDelta();
     int v;
-    if (e->delta() > 0)
+    if ((angleDelta.y() != 0 ? angleDelta.y() : angleDelta.x()) > 0)
         v = value() + wheelStep;
     else
         v = value() - wheelStep;
     v -= v % wheelStep;
     QSlider::setValue(v);
 }
-void Slider::enterEvent(QEvent *e)
+void Slider::enterEvent(Q_ENTER_EVENT *e)
 {
     lastMousePos = -1;
     QSlider::enterEvent(e);
