@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2023  Błażej Szczygieł
+    Copyright (C) 2010-2025  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -34,6 +34,9 @@
 #include <QPainter>
 #include <QLibrary>
 #include <QWidget>
+#ifdef Q_OS_WIN
+#   include <QOperatingSystemVersion>
+#endif
 
 /* OpenGL|ES 2.0 doesn't have those definitions */
 #ifndef GL_MAP_WRITE_BIT
@@ -51,6 +54,12 @@
 #ifndef GL_TEXTURE_RECTANGLE_ARB
     #define GL_TEXTURE_RECTANGLE_ARB 0x84F5
 #endif
+#ifndef GL_R16
+    #define GL_R16 GL_R16_EXT
+#endif
+#ifndef GL_RED
+    #define GL_RED GL_RED_EXT
+#endif
 
 OpenGLCommon::OpenGLCommon() :
     VideoOutputCommon(false),
@@ -60,23 +69,13 @@ OpenGLCommon::OpenGLCommon() :
     texCoordYCbCrLoc(-1), positionYCbCrLoc(-1), texCoordOSDLoc(-1), positionOSDLoc(-1),
     numPlanes(0),
     target(0),
-    hasPbo(false),
-    isPaused(false), isOK(false), hasImage(false), doReset(true), setMatrix(true), correctLinesize(false), canUseHueSharpness(true),
+    m_canUse16bitTexture(m_glInstance->canUse16bitTexture),
+    hasPbo(m_glInstance->hasPbo),
+    isPaused(false), isOK(false), hasImage(false), doReset(true), setMatrix(true), correctLinesize(false), m_gl3(true),
     outW(-1), outH(-1), verticesIdx(0),
-    hasVbo(true),
+    hasVbo(m_glInstance->hasVbo),
     nIndices(0)
 {
-#ifndef OPENGL_ES2
-    glActiveTexture = m_glInstance->glActiveTexture;
-    glGenBuffers = m_glInstance->glGenBuffers;
-    glBindBuffer = m_glInstance->glBindBuffer;
-    glBufferData = m_glInstance->glBufferData;
-    glDeleteBuffers = m_glInstance->glDeleteBuffers;
-#endif
-    glMapBufferRange = m_glInstance->glMapBufferRange;
-    glMapBuffer = m_glInstance->glMapBuffer;
-    glUnmapBuffer = m_glInstance->glUnmapBuffer;
-
     videoAdjustment.unset();
 
     /* Initialize texCoordYCbCr array */
@@ -84,7 +83,7 @@ OpenGLCommon::OpenGLCommon() :
     texCoordYCbCr[1] = texCoordYCbCr[3] = 1.0f;
 
 #ifndef Q_OS_MACOS
-    canUseHueSharpness = (m_glInstance->glVer >= 30);
+    m_gl3 = (m_glInstance->glVer >= 30);
 #endif
 
     m_matrixChangeFn = [this] {
@@ -190,7 +189,7 @@ void OpenGLCommon::initialize(const std::shared_ptr<OpenGLHWInterop> &hwInterop)
 #ifdef Q_OS_WIN
 void OpenGLCommon::setWindowsBypassCompositor(bool bypassCompositor)
 {
-    if (!bypassCompositor && QSysInfo::windowsVersion() <= QSysInfo::WV_6_1) // Windows 7 and Vista can disable DWM composition, so check it
+    if (!bypassCompositor && QOperatingSystemVersion::current() <= QOperatingSystemVersion::Windows7) // Windows 7 and Vista can disable DWM composition, so check it
     {
         using DwmIsCompositionEnabledProc = HRESULT (WINAPI *)(BOOL *pfEnabled);
         if (auto DwmIsCompositionEnabled = (DwmIsCompositionEnabledProc)GetProcAddress(GetModuleHandleA("dwmapi.dll"), "DwmIsCompositionEnabled"))
@@ -243,6 +242,16 @@ void OpenGLCommon::setTextureParameters(GLenum target, quint32 texture, GLint pa
 
 void OpenGLCommon::initializeGL()
 {
+    initializeOpenGLFunctions();
+
+#if !defined(QT_OPENGL_ES_2) && !defined(QT_FEATURE_opengles2)
+    if (hasPbo && !m_glInstance->hasMapBufferRange)
+    {
+        Q_ASSERT(m_glInstance->hasMapBuffer);
+        m_gl15.initializeOpenGLFunctions();
+    }
+#endif
+
     shaderProgramVideo.reset(new QOpenGLShaderProgram);
     shaderProgramOSD.reset(new QOpenGLShaderProgram);
 
@@ -252,22 +261,26 @@ void OpenGLCommon::initializeGL()
     if (numPlanes == 1)
     {
         videoFrag = readShader(":/opengl/VideoRGB.frag");
-        if (canUseHueSharpness)
+        if (m_gl3)
         {
             //Use sharpness only when OpenGL/OpenGL|ES version >= 3.0, because it can be slow on old hardware and/or buggy drivers and may increase CPU usage!
-            videoFrag.prepend("#define Sharpness\n");
+            videoFrag.prepend("#define GL3\n");
+            videoFrag.prepend(readShader(":/opengl/colorspace.glsl"));
         }
     }
     else
     {
         videoFrag = readShader(":/opengl/VideoYCbCr.frag");
-        if (canUseHueSharpness)
+        if (numPlanes == 2)
+        {
+            videoFrag.prepend("#define NV12\n");
+        }
+        if (m_gl3)
         {
             //Use hue and sharpness only when OpenGL/OpenGL|ES version >= 3.0, because it can be slow on old hardware and/or buggy drivers and may increase CPU usage!
-            videoFrag.prepend("#define HueAndSharpness\n");
+            videoFrag.prepend("#define GL3\n");
+            videoFrag.prepend(readShader(":/opengl/colorspace.glsl"));
         }
-        if (numPlanes == 2)
-            videoFrag.prepend("#define NV12\n");
     }
     if (target == GL_TEXTURE_RECTANGLE_ARB)
         videoFrag.prepend("#define TEXTURE_RECTANGLE\n");
@@ -363,6 +376,10 @@ void OpenGLCommon::paintGL()
             videoFrame.height(1),
             videoFrame.height(2),
         };
+        const int bytesMultiplier = (m_depth + 7) / 8;
+        const GLenum dataType = (bytesMultiplier == 1) ? GL_UNSIGNED_BYTE : GL_UNSIGNED_SHORT;
+        const GLint internalFmt = (bytesMultiplier == 1) ? GL_LUMINANCE : GL_R16;
+        const GLenum fmt = (bytesMultiplier == 1) ? GL_LUMINANCE : GL_RED;
 
         if (doReset)
         {
@@ -395,21 +412,21 @@ void OpenGLCommon::paintGL()
                 /* Prepare textures */
                 for (qint32 p = 0; p < 3; ++p)
                 {
-                    const GLsizei w = correctLinesize ? videoFrame.linesize(p) : widths[p];
+                    const GLsizei w = correctLinesize ? videoFrame.linesize(p) / bytesMultiplier : widths[p];
                     const GLsizei h = heights[p];
                     if (p == 0)
                         m_textureSize = QSize(w, h);
                     if (hasPbo)
                     {
                         glBindBuffer(GL_PIXEL_UNPACK_BUFFER, pbo[p + 1]);
-                        glBufferData(GL_PIXEL_UNPACK_BUFFER, w * h, nullptr, GL_DYNAMIC_DRAW);
+                        glBufferData(GL_PIXEL_UNPACK_BUFFER, w * h * bytesMultiplier, nullptr, GL_DYNAMIC_DRAW);
                     }
                     glBindTexture(GL_TEXTURE_2D, textures[p + 1]);
-                    glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, w, h, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, nullptr);
+                    glTexImage2D(GL_TEXTURE_2D, 0, internalFmt, w, h, 0, fmt, dataType, nullptr);
                 }
 
                 /* Prepare texture coordinates */
-                texCoordYCbCr[2] = texCoordYCbCr[6] = (videoFrame.linesize(0) == widths[0]) ? 1.0f : (widths[0] / (videoFrame.linesize(0) + 1.0f));
+                texCoordYCbCr[2] = texCoordYCbCr[6] = (videoFrame.linesize(0) / bytesMultiplier == widths[0]) ? 1.0f : (widths[0] / (videoFrame.linesize(0) / bytesMultiplier + 1.0f));
             }
             resetDone = true;
             hasImage = false;
@@ -443,27 +460,29 @@ void OpenGLCommon::paintGL()
             for (qint32 p = 0; p < 3; ++p)
             {
                 const quint8 *data = videoFrame.constData(p);
-                const GLsizei w = correctLinesize ? videoFrame.linesize(p) : widths[p];
+                const GLsizei w = correctLinesize ? videoFrame.linesize(p) / bytesMultiplier : widths[p];
                 const GLsizei h = heights[p];
                 if (hasPbo)
                 {
                     glBindBuffer(GL_PIXEL_UNPACK_BUFFER, pbo[p + 1]);
-                    quint8 *dst;
-                    if (glMapBufferRange)
-                        dst = (quint8 *)glMapBufferRange(GL_PIXEL_UNPACK_BUFFER, 0, w * h, GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT);
+                    quint8 *dst = nullptr;
+                    if (m_glInstance->hasMapBufferRange)
+                        dst = (quint8 *)glMapBufferRange(GL_PIXEL_UNPACK_BUFFER, 0, w * h * bytesMultiplier, GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT);
+#if !defined(QT_OPENGL_ES_2) && !defined(QT_FEATURE_opengles2)
                     else
-                        dst = (quint8 *)glMapBuffer(GL_PIXEL_UNPACK_BUFFER, GL_WRITE_ONLY);
+                        dst = (quint8 *)m_gl15.glMapBuffer(GL_PIXEL_UNPACK_BUFFER, GL_WRITE_ONLY);
+#endif
                     if (!dst)
                         glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
                     else
                     {
                         if (correctLinesize)
-                            memcpy(dst, data, w * h);
+                            memcpy(dst, data, w * h * bytesMultiplier);
                         else for (int y = 0; y < h; ++y)
                         {
-                            memcpy(dst, data, w);
+                            memcpy(dst, data, w * bytesMultiplier);
                             data += videoFrame.linesize(p);
-                            dst  += w;
+                            dst  += w * bytesMultiplier;
                         }
                         glUnmapBuffer(GL_PIXEL_UNPACK_BUFFER);
                         data = nullptr;
@@ -472,10 +491,10 @@ void OpenGLCommon::paintGL()
                 glActiveTexture(GL_TEXTURE0 + p);
                 glBindTexture(GL_TEXTURE_2D, textures[p + 1]);
                 if (hasPbo || correctLinesize)
-                    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, w, h, GL_LUMINANCE, GL_UNSIGNED_BYTE, data);
+                    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, w, h, fmt, dataType, data);
                 else for (int y = 0; y < h; ++y)
                 {
-                    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, y, w, 1, GL_LUMINANCE, GL_UNSIGNED_BYTE, data);
+                    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, y, w, 1, fmt, dataType, data);
                     data += videoFrame.linesize(p);
                 }
             }
@@ -543,7 +562,22 @@ void OpenGLCommon::paintGL()
             const float hue = videoAdjustment.hue / -31.831f;
             shaderProgramVideo->setUniformValue("uVideoEq", brightness, contrast, saturation, hue);
             shaderProgramVideo->setUniformValue("uSharpness", sharpness);
+            shaderProgramVideo->setUniformValue("uBitsMultiplier", m_bitsMultiplier);
         }
+
+        if (m_colorPrimaries == AVCOL_PRI_BT2020 && (m_colorTrc == AVCOL_TRC_BT709 || m_colorTrc == AVCOL_TRC_SMPTE2084 || m_colorTrc == AVCOL_TRC_ARIB_STD_B67))
+        {
+            shaderProgramVideo->setUniformValue("uTrc", m_colorTrc);
+            shaderProgramVideo->setUniformValue("uMaxLuminance", m_maxLuminance);
+            shaderProgramVideo->setUniformValue("uColorPrimariesMatrix", Functions::getColorPrimariesTo709Matrix(m_colorPrimaries).toGenericMatrix<3, 3>());
+        }
+        else
+        {
+            shaderProgramVideo->setUniformValue("uTrc", 0);
+        }
+
+        shaderProgramVideo->setUniformValue("uNegative", static_cast<int>(videoAdjustment.negative));
+
         shaderProgramVideo->setUniformValue("uTextureSize", m_textureSize);
 
         doReset = !resetDone;
@@ -602,11 +636,13 @@ void OpenGLCommon::paintGL()
                 glBindBuffer(GL_PIXEL_UNPACK_BUFFER, pbo[0]);
                 if (hasNewSize)
                     glBufferData(GL_PIXEL_UNPACK_BUFFER, dataSize, nullptr, GL_DYNAMIC_DRAW);
-                quint8 *dst;
-                if (glMapBufferRange)
+                quint8 *dst = nullptr;
+                if (m_glInstance->hasMapBufferRange)
                     dst = (quint8 *)glMapBufferRange(GL_PIXEL_UNPACK_BUFFER, 0, dataSize, GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT);
+#if !defined(QT_OPENGL_ES_2) && !defined(QT_FEATURE_opengles2)
                 else
-                    dst = (quint8 *)glMapBuffer(GL_PIXEL_UNPACK_BUFFER, GL_WRITE_ONLY);
+                    dst = (quint8 *)m_gl15.glMapBuffer(GL_PIXEL_UNPACK_BUFFER, GL_WRITE_ONLY);
+#endif
                 if (!dst)
                     glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
                 else
@@ -680,17 +716,12 @@ QByteArray OpenGLCommon::readShader(const QString &fileName, bool pure)
     QByteArray shader;
     if (!pure)
     {
-#ifdef OPENGL_ES2
-        shader = "precision highp float;\n";
-#endif
+        if (m_glInstance->isGLES)
+            shader = "precision highp float;\n";
         shader.append("#line 1\n");
     }
-#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
     const auto data = res.uncompressedData();
     shader.append(data.constData(), data.size());
-#else
-    shader.append((const char *)res.data(), res.size());
-#endif
     return shader;
 }
 
